
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

class AttestationConstants:
    MANUAL_POLICY_RG_ASSIGNMENT = "7a00c0f2284a40128f1d3d22"
    MANUAL_POLICY_RG_INITIATIVE_ASSIGNMENT = "e148d7cc6585407bb1638382"
    MANUAL_POLICY_RG_INITIATIVE_REFID = "cliTests:RGManualPolicy_1"

    MANUAL_POLICY_SUB_ASSIGNMENT = "e861d6390e834761982c1c25"
    MANUAL_POLICY_SUB_INITIATIVE_ASSIGNMENT = "e1932be59a1c4b97bde62716"
    MANUAL_POLICY_SUB_INITIATIVE_REFID = "cliTests:SubscriptionManualPolicy_1"

    MANUAL_POLICY_RESOURCE_ASSIGNMENT = "ccbf75fff357452e88ca5532"
    MANUAL_POLICY_RESOURCE_INITIATIVE_ASSIGNMENT = "1213836084664464920a1b65"
    MANUAL_POLICY_RESOURCE_INITIATIVE_REFID = "cliTests:ResourceManualPolicy_1"

    ATTESTATION_TEST_RESOURCE_NAME = "CLI-TEST-VN"
    ATTESTATION_TEST_RG_NAME = "cli_test_attestations"
    ATTESTATION_RESOURCE_TYPE = "/providers/microsoft.policyinsights/attestations"
